from typing import Optional

from pydantic import BaseModel, Field

from .comment import Comment
from .resource import Resource


class Snippet(BaseModel):
    channel_id: str
    video_id: str
    top_level_comment: Comment
    can_reply: bool
    total_reply_count: int
    is_public: bool


class CommentThread(Resource):
    id: str
    snippet: Snippet
    replies: list[Comment] = Field(default_factory=list)



from datetime import datetime
from typing import Optional

from pydantic import Field

from .youtube_request import Filter, OptionalParameters, Part


class SubscriptionPart(Part):
    part: list[str] = Field(
        default=['contentDetails', 'id', 'snippet', 'subscriberSnippet']
    )


class SubscriptionFilter(Filter):
    channelId: Optional[str] = None
    mine: Optional[bool] = None
    id: Optional[bool] = None
    myRecentSubscribers: Optional[bool] = None
    mySubscribers: Optional[bool] = None


class SubscriptionOptionalParameters(OptionalParameters):
    forChannelId: Optional[str] = None
    order: Optional[str] = None
    pageToken: Optional[str] = None
    maxResults: Optional[int] = None



from typing import Any

from ...models import Comment, CommentSnippet
from ..resource import YouTubeResource


class CommentResource(YouTubeResource):
    def __init__(self, youtube_client: Any) -> None:
        super().__init__(youtube_client)

    def parse_author(self, comment_snippet: dict) -> dict:
        author: dict = dict()
        author['display_name'] = comment_snippet['authorDisplayName']
        author['profile_image_url'] = comment_snippet['authorProfileImageUrl']
        author['channel_url'] = comment_snippet['authorChannelUrl']
        author['channel_id'] = comment_snippet['authorChannelId']['value']
        return author

    def parse_snippet(self, snippet: dict) -> CommentSnippet:
        """Parse comment snippet"""
        parsed_snippet: dict[str, Any] = dict()
        parsed_snippet['channel_id'] = snippet['channelId']
        parsed_snippet['text_original'] = snippet['textOriginal']
        parsed_snippet['text_display'] = snippet['textDisplay']
        parsed_snippet['author'] = self.parse_author(snippet)
        parsed_snippet['can_rate'] = snippet['canRate']
        parsed_snippet['viewer_rating'] = snippet['viewerRating']
        parsed_snippet['like_count'] = snippet['likeCount']
        parsed_snippet['updated_at'] = snippet['updatedAt']
        parsed_snippet['published_at'] = snippet['publishedAt']
        parsed_snippet['parent_id'] = snippet.get('parentId', None)
        return CommentSnippet(**parsed_snippet)

    def parse_item(self, item: dict) -> Comment:
        id_data: dict = self.parse_id(item)
        snippet_data: dict = dict(snippet=self.parse_snippet(item['snippet']))
        id_data.update(snippet_data)
        return Comment(**id_data)



from .activity import Activity, ActivityContentDetails, ActivitySnippet
from .activity_type import (
    Activities,
    ActivityType,
    Bulletin,
    ChannelItem,
    CommentAdded,
    PromotedItem,
    Reccomendation,
    ResourceId,
    ShareToSocial,
    UserSubscribedToChannel,
    VideoAddedToFavorite,
    VideoAddedToPlaylist,
    VideoLiked,
    VideoUploaded,
)
from .caption import Caption
from .channel import (
    Channel,
    ChannelContentDetails,
    ChannelSnippet,
    ChannelStatistics,
    ChannelStatus,
)
from .channel_banner import ChannelBanner
from .channel_section import ChannelSection, ChannelSectionType
from .comment import Author, Comment, CommentSnippet
from .comment_thread import CommentThread
from .content_details import BaseContentDetails
from .language import Language, LanguageSnippet
from .localized import Localized
from .playlist import Playlist
from .playlist_item import PlaylistItem
from .region import Region, RegionSnippet
from .resource import Resource
from .resource_type import ResourceType
from .scopes import Scopes
from .search import Search
from .snippet import BaseSnippet
from .status import BaseStatus
from .subscription import (
    SubscriberSnippet,
    Subscription,
    SubscriptionContentDetails,
    SubscriptionSnippet,
)
from .thumbnail import Thumbnail, ThumbnailResolution
from .video import ContentDetails, Statistics, Status, Video
from .video_abuse_report_reason import VideoAbuseReportReason, VideoAbuseReportReasonSnippet
from .video_category import VideoCategory, VideoCategorySnippet
from .watermark import WaterMark



from datetime import datetime
from enum import Enum, auto

from pydantic import BaseModel


class TrackKind(Enum):
    Asr = auto()
    Forced = auto()
    Standard = auto()


class AudioTrackType(Enum):
    Commentary = auto()
    Descriptive = auto()
    Primary = auto()
    Unknown = auto()


class Status(Enum):
    Failed = auto()
    Serving = auto()
    Syncing = auto()


class FailureReason(Enum):
    ProcessingFailed = auto()
    UnknownFormat = auto()
    UnsupportedFormat = auto()


class Caption(BaseModel):
    video_id: str
    last_updated: datetime
    track_kind: TrackKind
    language: str
    name: str
    audio_track_type: str
    is_cc: bool
    is_large: bool
    is_easy_reader: bool
    is_draft: bool
    is_auto_synced: bool
    status: str
    failure_reason: str



from typing import Any

import googleapiclient.errors

from youtube.models import Resource

from ...exceptions import SubscriptionDuplicate, SubscriptionForbidden, SubscriptionNotFound
from ...models import BaseContentDetails, BaseSnippet, Subscription, SubscriptionSnippet
from ...schemas import (
    SubscriptionFilter,
    SubscriptionOptionalParameters,
    SubscriptionPart,
    YouTubeRequest,
    YouTubeResponse,
)
from ..resource import YouTubeResource


class YouTubeSubscription(YouTubeResource):
    def __init__(self, youtube_client: Any) -> None:
        super().__init__(youtube_client)

    def parse_snippet(self, snippet_data: dict[str, Any]) -> SubscriptionSnippet:
        base_snippet: BaseSnippet = self.parse_base_snippet(snippet_data)

    def parse_item(self, item: dict) -> Resource:
        return super().parse_item(item)

    def list_channel_subscriptions(self, request: YouTubeRequest) -> YouTubeResponse:
        request_dict = self.create_request_dict(request)
        actvities_request = self.youtube_client.subscriptions().list(**request_dict)
        actvities_response = actvities_request.execute()
        return actvities_response

    def list_my_subscriptions(self) -> YouTubeResponse:
        filter: SubscriptionFilter = SubscriptionFilter(mine=True)
        part: SubscriptionPart = SubscriptionPart()
        optional_params: SubscriptionOptionalParameters = (
            SubscriptionOptionalParameters()
        )
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=part, filter=filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(youtube_request)
        subscriptions_request = self.youtube_client.subscriptions().list(**request_dict)
        subscriptions_response = subscriptions_request.execute()
        return subscriptions_response

    def do_i_subscribe(self, channel_id: str) -> bool:
        filter: SubscriptionFilter = SubscriptionFilter(mine=True)
        part: SubscriptionPart = SubscriptionPart()
        optional_params: SubscriptionOptionalParameters = (
            SubscriptionOptionalParameters()
        )
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=part, filter=filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(youtube_request)
        actvities_request = self.youtube_client.subscriptions().list(**request_dict)
        actvities_response = actvities_request.execute()
        return actvities_response

    def is_subscribed(self, channel_id: str, user_channel: str) -> bool:
        filter: SubscriptionFilter = SubscriptionFilter(channelId=user_channel)
        part: SubscriptionPart = SubscriptionPart()
        optional_params: SubscriptionOptionalParameters = (
            SubscriptionOptionalParameters(forChannelId=channel_id)
        )
        youtube_request: YouTubeRequest = YouTubeRequest(
            part=part, filter=filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(youtube_request)
        actvities_request = self.youtube_client.subscriptions().list(**request_dict)
        actvities_response = actvities_request.execute()
        return actvities_response

    def subscribe_to_channel(self, channel_id: str) -> Subscription:
        try:
            subscription_request = self.youtube_client.subscriptions().insert(
                part=','.join(SubscriptionPart().part),
                body={
                    'snippet': {
                        'resourceId': {
                            'kind': 'youtube#channel',
                            'channelId': channel_id,
                        }
                    }
                },
            )
            subscription_response = subscription_request.execute()
        except googleapiclient.errors.HttpError as e:
            error_str: str = e.error_details[0]['reason']
            if error_str == 'subscriptionDuplicate':
                raise SubscriptionDuplicate('That subscription already exists!')
        return subscription_response

    def unsubscribe_from_channel(self, subscription_id: str) -> None:
        try:
            unsubscribe_request = self.youtube_client.subscriptions().delete(
                id=subscription_id
            )
            unsubscribe_request.execute()
        except googleapiclient.errors.HttpError as e:
            error_str: str = e.error_details[0]['reason']
            if error_str == 'subscriptionNotFound':
                raise SubscriptionNotFound('That subscription was not found.')



from typing import Any, Iterator

from ...models import BaseSnippet, Playlist
from ...schemas import (
    CreatePlaylist,
    PlaylistFilter,
    PlaylistOptionalParameters,
    PlaylistPart,
    YouTubeListResponse,
    YouTubeRequest,
    YouTubeResponse,
)
from ..resource import YouTubeResource


class YouTubePlaylist(YouTubeResource):
    def __init__(self, youtube_client: Any) -> None:
        super().__init__(youtube_client)
        self.my_playlists_schema: YouTubeRequest = None
        self.my_playlists_nxt_page_tkn: str = None

    def parse_snippet(self, snippet_data: dict[str, Any]) -> dict[str, Any]:
        base_snippet: BaseSnippet = self.parse_base_snippet(snippet_data)
        parsed_snippet: dict[str, Any] = base_snippet.model_dump()
        parsed_snippet['channel_id'] = snippet_data['channelId']
        parsed_snippet['channel_title'] = snippet_data['channelTitle']
        if snippet_data.get('defaultLanguage'):
            parsed_snippet['default_language'] = snippet_data['defaultLanguage']
        if snippet_data.get('localized'):
            parsed_snippet['localized'] = self.parse_localizations(
                snippet_data['localized']
            )
        if snippet_data.get('defaultAudioLanguage'):
            parsed_snippet['default_audio_language'] = snippet_data[
                'defaultAudioLanguage'
            ]
        return dict(snippet=parsed_snippet)

    def parse_status(self, status: dict) -> dict:
        parsed_status: dict[str, str] = {}
        parsed_status['privacy_status'] = status['privacyStatus']
        return dict(status=parsed_status)

    def parse_content_details(self, content_details: dict) -> dict:
        parsed_content_details: dict = dict()
        parsed_content_details['item_count'] = content_details['itemCount']
        return dict(content_details=parsed_content_details)

    def parse_item(self, item: dict) -> Playlist:
        id_data: dict = self.parse_id(item)
        if item.get('snippet'):
            snippet_data: dict = self.parse_snippet(item['snippet'])
            id_data.update(snippet_data)
        if item.get('contentDetails'):
            content_details: dict = self.parse_content_details(item['contentDetails'])
            id_data.update(content_details)
        if item.get('status'):
            status: dict = self.parse_status(item['status'])
            id_data.update(status)
        return Playlist(**id_data)

    def find_channel_playlists(
        self, channel_id: str, max_result: int = 25
    ) -> YouTubeListResponse:
        part: PlaylistPart = PlaylistPart()
        request_filter: PlaylistFilter = PlaylistFilter(channelId=channel_id)
        optional_params: PlaylistOptionalParameters = PlaylistOptionalParameters(
            maxResults=max_result
        )
        request_schema: YouTubeRequest = YouTubeRequest(
            part=part, filter=request_filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(request_schema)
        find_channel_playlist_request: dict = self.youtube_client.playlists().list(
            **request_dict
        )
        find_channel_playlist_result: dict[
            str, int | str
        ] = find_channel_playlist_request.execute()
        return self.parse_youtube_list_response(find_channel_playlist_result)

    def find_my_playlists(self, max_result: int = 25) -> YouTubeListResponse:
        part: PlaylistPart = PlaylistPart()
        request_filter: PlaylistFilter = PlaylistFilter(mine=True)
        optional_params: PlaylistOptionalParameters = PlaylistOptionalParameters(
            maxResults=max_result
        )
        request_schema: YouTubeRequest = YouTubeRequest(
            part=part,
            optional_parameters=optional_params,
            filter=request_filter,
        )
        request_dict = self.create_request_dict(request_schema)
        find_channel_playlist_request: dict = self.youtube_client.playlists().list(
            **request_dict
        )
        find_channel_playlist_result: dict[
            str, int | str
        ] = find_channel_playlist_request.execute()
        return self.parse_youtube_response(find_channel_playlist_result)

    def insert_playlist(self, playlist_schema: CreatePlaylist) -> Playlist:
        create_playlist_req = self.youtube_client.playlists().insert(
            part=','.join(playlist_schema.model_dump().keys()),
            body=playlist_schema.model_dump(),
        )
        create_playlist_resp = create_playlist_req.execute()
        return self.parse_item(create_playlist_resp)

    def update_playlist(
        self, playlist_id: str, playlist_schema: CreatePlaylist
    ) -> Playlist:
        request_dict: dict = dict(id=playlist_id)
        request_dict.update(playlist_schema.model_dump())
        update_playlist_req = self.youtube_client.playlists().update(
            part=','.join(playlist_schema.model_dump().keys()), body=request_dict
        )
        update_playlist_resp = update_playlist_req.execute()
        return self.parse_item(update_playlist_resp)

    def delete_playlist(self, playlist_id: str) -> None:
        delete_playlist_req = self.youtube_client.playlists().delete(id=playlist_id)
        delete_playlist_req.execute()

    def __iter__(self):
        return self

    def __next__(self) -> list[Playlist]:
        self.my_playlists_schema.optional_parameters.pageToken = (
            self.my_playlists_nxt_page_tkn
        )
        request_dict = self.create_request_dict(self.my_playlists_schema)
        find_channel_playlist_request: dict = self.youtube_client.playlists().list(
            **request_dict
        )
        find_channel_resp: dict = find_channel_playlist_request.execute()
        find_channel_playlist_result: YouTubeResponse = self.parse_youtube_response(
            find_channel_resp
        )
        self.my_playlists_nxt_page_tkn = find_channel_playlist_result.nextPageToken
        if not self.my_playlists_nxt_page_tkn:
            raise StopIteration()
        return find_channel_playlist_result.items

    def get_my_playlists_iterator(self, max_results: int = 10) -> Iterator:
        part: PlaylistPart = PlaylistPart()
        request_filter: PlaylistFilter = PlaylistFilter(mine=True)
        optional_params: PlaylistOptionalParameters = PlaylistOptionalParameters(
            maxResults=max_results
        )
        self.my_playlists_schema = YouTubeRequest(
            part=part,
            optional_parameters=optional_params,
            filter=request_filter,
        )
        return self



from typing import Any

from ...models import (
    Activities,
    Activity,
    ActivityContentDetails,
    ActivitySnippet,
    ActivityType,
    Bulletin,
    ChannelItem,
    CommentAdded,
    PromotedItem,
    Reccomendation,
    ResourceId,
    ShareToSocial,
    UserSubscribedToChannel,
    VideoAddedToFavorite,
    VideoAddedToPlaylist,
    VideoLiked,
    VideoUploaded,
)
from ...schemas import (
    ActivityFilter,
    ActivityOptionalParameters,
    ActivityPart,
    YouTubeRequest,
    YouTubeResponse,
    YouTubeListResponse
)
from ..resource import YouTubeResource


class YouTubeActivity(YouTubeResource):
    def __init__(self, youtube_client: Any) -> None:
        super().__init__(youtube_client)

    def list_channel_activity(self, request: YouTubeRequest) -> YouTubeResponse:
        raise NotImplementedError()

    def list_my_activities(self) -> YouTubeListResponse:
        actvities_request = self.youtube_client.activities().list(
            part='id,snippet,contentDetails', maxResults=25, mine=True
        )
        actvities_response = actvities_request.execute()
        return actvities_response

    def parse_snippet(self, snippet_data: dict[str, Any]) -> ActivitySnippet:
        snippet: dict = dict(**self.parse_base_snippet(snippet_data).model_dump())
        snippet['channel_id'] = snippet_data['channelId']
        snippet['channel_title'] = snippet_data['channelTitle']
        snippet['type'] = snippet_data['type']
        if snippet_data.get('groupId'):
            snippet['group_id'] = snippet_data['groupId']
        else:
            snippet['group_id'] = None
        return ActivitySnippet(**snippet)

    def parse_activity_type(
        self, content_details: dict, activities_type: str
    ) -> ActivityType:
        activity_type: ActivityType = None
        if activities_type == Activities.playlistItem.value:
            activity_type = VideoAddedToPlaylist(
                video_id=content_details['playlistItem']['resourceId']['videoId'],
                playlist_id=content_details['playlistItem']['playlistId'],
            )
        elif activities_type == Activities.channelItem.value:
            activity_type = ChannelItem(
                resource_id=ResourceId(
                    kind=content_details['recommendation']['resourceId']['kind'],
                    id=content_details['recommendation']['resourceId']['channelId'],
                )
            )
        elif activities_type == Activities.comment.value:
            id: str = None
            kind = content_details['comment']['resourceId']['kind']
            if kind == 'youtube#video':
                id = 'videoId'
            elif kind == 'youtube#channel':
                id = 'channelId'
            activity_type = CommentAdded(
                resource_id=ResourceId(
                    kind=kind, id=content_details['recommendation']['resourceId'][id]
                )
            )
        elif activities_type == Activities.favorite.value:
            activity_type = VideoAddedToFavorite(
                resource_id=ResourceId(
                    kind=content_details['recommendation']['resourceId']['kind'],
                    id=content_details['recommendation']['resourceId']['videoId'],
                )
            )
        elif activities_type == Activities.like.value:
            activity_type = VideoLiked(
                resource_id=ResourceId(
                    kind=content_details['recommendation']['resourceId']['kind'],
                    id=content_details['recommendation']['resourceId']['videoId'],
                )
            )
        elif activities_type == Activities.promotedItem.value:
            activity_type = PromotedItem()
        elif activities_type == Activities.recommendation.value:
            activity_type = Reccomendation(
                reason='',
                resource_id=ResourceId(kind='', id=''),
                seed_resource_id=ResourceId(kind='', id=''),
            )
        elif activities_type == Activities.social.value:
            activity_type = ShareToSocial(
                type='',
                resource_id=ResourceId(kind='', id=''),
                author='',
                reference_url='',
                image_url='',
            )
        elif activities_type == Activities.subscription.value:
            activity_type = UserSubscribedToChannel(
                resource_id=ResourceId(
                    kind=content_details['recommendation']['resourceId']['kind'],
                    id=content_details['recommendation']['resourceId']['channelId'],
                )
            )
        elif activities_type == Activities.upload.value:
            activity_type = VideoUploaded(video_id='')
        elif activities_type == Activities.bulletin.value:
            activity_type = Bulletin()
        else:
            activity_type = ActivityType()
        return activity_type

    def parse_content_details(
        self, content_details_data: dict, activities_type: str
    ) -> ActivityContentDetails:
        content_details: dict = dict(
            **self.parse_base_content_details(content_details_data).model_dump()
        )
        content_details['activity_type'] = self.parse_activity_type(
            content_details=content_details_data, activities_type=activities_type
        )
        return ActivityContentDetails(**content_details)

    def parse_item(self, item: dict) -> Activity:
        activity_data: dict = dict()
        activity_data.update(self.parse_id(item))
        activity_data.update(dict(snippet=self.parse_snippet(item['snippet'])))
        activity_data.update(
            dict(
                content_details=self.parse_content_details(
                    content_details_data=item['contentDetails'],
                    activities_type=item['snippet']['type'],
                )
            )
        )
        activity: Activity = Activity(**activity_data)
        return activity

    def parse_items(self, activity_items: list[Activity]):
        parsed_activity_items: list[Activity] = [
            self.parse_item(activity_item) for activity_item in activity_items
        ]
        return parsed_activity_items



from pydantic import BaseModel

from .resource import Resource


class VideoAbuseReportReasonSnippet(BaseModel):
    label: str


class VideoAbuseReportReason(Resource):
    snippet: VideoAbuseReportReasonSnippet



from abc import ABC, abstractmethod
from typing import Any, Generic, TypeVar

T = TypeVar('T')

from ..models import (
    BaseContentDetails,
    BaseSnippet,
    BaseStatus,
    Resource,
    Thumbnail,
    ThumbnailResolution,
)
from ..schemas import (
    Filter,
    OptionalParameters,
    PageInfo,
    Part,
    YouTubeListResponse,
    YouTubeRequest,
    YouTubeResponse,
)


class YouTubeResource(ABC):
    def __init__(self, youtube_client: Any) -> None:
        self.youtube_client: Any = youtube_client

    def generate_part(self, part: Part) -> str:
        """Generate the 'Part' for a request.

        Parameters
        ----------
        part: Part
            An instance of part that contains all the required details.
        """
        part_str: str = ','.join(part.part)
        return part_str

    def generate_optional_parameters(self, optional_params: OptionalParameters) -> dict:
        """Generate Optional parmateers for a request.

        Parameters
        ----------
        optional_parameters: OptionalParameters
            Optional parameters used when sending the request
        """
        optional: dict[str, Any] = dict()
        for key, value in optional_params.model_dump().items():
            #  We only want optional parameters that were provided
            if value:
                optional[key] = value
        #  This is useful when sending search request and type is provided as a list i.e ['video']
        if optional_params.model_dump().get('type'):
            optional['type'] = ','.join(optional_params.type)
        return optional

    def generate_filter(self, request_filter: Filter) -> dict[str, Any]:
        """Generate the filters for request to youtube."""
        #  We pick the very first filter.
        for key, value in request_filter.model_dump().items():
            if value:
                if key == 'id':
                    value: str = ','.join(value)
                return {key: value}
        return {}

    def create_request_dict(self, request_schema: YouTubeRequest) -> dict[str, Any]:
        """Create the request dict for sending request to youtube."""
        request_dict: dict[str, Any] = dict()
        request_dict['part'] = self.generate_part(request_schema.part)
        request_dict.update(
            self.generate_optional_parameters(request_schema.optional_parameters)
        )
        request_dict.update(self.generate_filter(request_schema.filter))
        return request_dict

    def parse_id(self, item: dict) -> dict:
        """Get the resource id from the youtube response."""
        return dict(id=item['id'])

    def parse_thumbnail(
        self, thumbnail_data: dict[str, int | str]
    ) -> ThumbnailResolution:
        """Parse a single thumbnail resolution.

        YouTube Thumbnails come in different resolutions e.g default, medium, high, standard
        """
        if not thumbnail_data:
            return None
        parsed_thumbnail: ThumbnailResolution = ThumbnailResolution(
            url=thumbnail_data.get('url', ''),
            width=thumbnail_data.get('width', 0),
            height=thumbnail_data.get('height', 0),
        )
        return parsed_thumbnail

    def parse_thumbnails(self, thumbnails_data: dict[str, int | str]) -> Thumbnail:
        """Parse all the thumbnails for a given resource."""
        parsed_thumbnail: Thumbnail = Thumbnail(
            default=self.parse_thumbnail(thumbnails_data.get('default', '')),
            medium=self.parse_thumbnail(thumbnails_data.get('medium', '')),
            high=self.parse_thumbnail(thumbnails_data.get('high', '')),
            standard=self.parse_thumbnail(thumbnails_data.get('standard', '')),
        )
        return parsed_thumbnail

    def parse_localizations(self, localized_details: dict) -> dict:
        """Parse localization details for a given resource."""
        parsed_localized: dict[str, str] = dict()
        parsed_localized['title'] = localized_details['title']
        parsed_localized['description'] = localized_details['description']
        return parsed_localized

    def parse_base_status(self, status: dict) -> BaseStatus:
        parsed_status: dict[str, str] = {}
        parsed_status['privacy_status'] = status['privacyStatus']
        return BaseStatus(**parsed_status)

    def parse_base_snippet(self, snippet_data: dict) -> BaseSnippet:
        parsed_snippet: dict[str, Any] = dict()
        parsed_snippet['published_at'] = snippet_data['publishedAt']
        parsed_snippet['title'] = snippet_data['title']
        parsed_snippet['description'] = snippet_data['description']
        parsed_snippet['thumbnails'] = self.parse_thumbnails(snippet_data['thumbnails'])
        return BaseSnippet(**parsed_snippet)

    def parse_base_content_details(self, content_details: dict) -> BaseContentDetails:
        return BaseContentDetails()

    @abstractmethod
    def parse_item(self, item: dict) -> Resource:
        pass

    def parse_items(self, items: list[dict]) -> list[Resource]:
        parsed_items: list[Resource] = [self.parse_item(item) for item in items]
        return parsed_items

    # Removed pageInfo=PageInfo(**youtube_list_response['pageInfo']),
    def parse_youtube_list_response(
        self, youtube_list_response: dict
    ) -> YouTubeListResponse:
        youtube_result: YouTubeListResponse = YouTubeListResponse(
            kind=youtube_list_response['kind'],
            etag=youtube_list_response['etag'],
            items=self.parse_items(youtube_list_response['items']),
        )
        return youtube_result

    def parse_youtube_response(self, youtube_response: dict) -> YouTubeResponse:
        youtube_result: YouTubeResponse = YouTubeResponse(
            kind=youtube_response['kind'],
            etag=youtube_response['etag'],
            pageInfo=PageInfo(**youtube_response['pageInfo']),
            items=self.parse_items(youtube_response['items']),
            nextPageToken=youtube_response.get('nextPageToken', ''),
            prevPageToken=youtube_response.get('prevPageToken', ''),
        )
        return youtube_result



from datetime import datetime
from typing import Optional

from pydantic import Field

from .youtube_request import Filter, OptionalParameters, Part


class ActivityPart(Part):
    part: list[str] = Field(default=['contentDetails', 'id', 'snippet'])


class ActivityFilter(Filter):
    channelId: Optional[str] = None
    mine: Optional[bool] = None


class ActivityOptionalParameters(OptionalParameters):
    regionCode: Optional[str] = None
    publishedAfter: Optional[datetime] = None
    publishedBefore: Optional[datetime] = None
    pageToken: Optional[str] = None
    maxResults: Optional[int] = None



from typing import Any

from ...models import BaseContentDetails, BaseSnippet, PlaylistItem
from ...schemas import (
    CreatePlaylistItem,
    PlaylistItemFilter,
    PlaylistItemOptionalParameters,
    PlaylistItemPart,
    YouTubeListResponse,
    YouTubeRequest,
    YouTubeResponse,
)
from ..resource import YouTubeResource


class YouTubePlaylistItem(YouTubeResource):
    def __init__(self, youtube_client: Any) -> None:
        super().__init__(youtube_client)

    def parse_resource_id(self, resource_id: dict) -> dict:
        parsed_resource_id: dict = dict()
        if resource_id.get('videoId'):
            parsed_resource_id['id'] = resource_id['videoId']
        elif resource_id.get('channelId'):
            parsed_resource_id['id'] = resource_id['channelId']
        else:
            parsed_resource_id['id'] = resource_id['playlistId']
        parsed_resource_id['kind'] = resource_id['kind']
        return parsed_resource_id

    def parse_snippet(self, snippet_data: dict[str, Any]) -> dict[str, Any]:
        base_snippet: BaseSnippet = self.parse_base_snippet(snippet_data)
        parsed_snippet: dict[str, Any] = base_snippet.model_dump()
        parsed_snippet['channel_id'] = snippet_data['channelId']
        parsed_snippet['channel_title'] = snippet_data['channelTitle']
        if snippet_data.get('videoOwnerChannelTitle'):
            parsed_snippet['video_owner_channel_title'] = snippet_data[
                'videoOwnerChannelTitle'
            ]
        if snippet_data.get('videoOwnerChannelTitle'):
            parsed_snippet['video_owner_channel_id'] = snippet_data[
                'videoOwnerChannelTitle'
            ]
        if snippet_data.get('playlistId'):
            parsed_snippet['playlist_id'] = snippet_data['playlistId']
        if snippet_data.get('position'):
            parsed_snippet['position'] = snippet_data['position']
        if snippet_data.get('playlistId'):
            parsed_snippet['resource_id'] = self.parse_resource_id(
                snippet_data['resourceId']
            )
        return dict(snippet=parsed_snippet)

    def parse_content_details(self, content_details: dict) -> dict:
        base_content_details: BaseContentDetails = self.parse_base_content_details(
            content_details
        )
        parsed_content_details: dict = base_content_details.model_dump()
        parsed_content_details['video_id'] = content_details['videoId']
        if content_details.get('note'):
            parsed_content_details['note'] = content_details['note']
        if content_details.get('videoPublishedAt'):
            parsed_content_details['video_published_at'] = content_details[
                'videoPublishedAt'
            ]
        return dict(content_details=parsed_content_details)

    def parse_item(self, item: dict) -> PlaylistItem:
        id_data: dict = self.parse_id(item)
        if item.get('snippet'):
            snippet_data: dict = self.parse_snippet(item['snippet'])
            id_data.update(snippet_data)
        if item.get('contentDetails'):
            content_details: dict = self.parse_content_details(item['contentDetails'])
            id_data.update(content_details)
        return PlaylistItem(**id_data)

    def find_playlist_items(
        self, playlist_id: str, max_results: int
    ) -> YouTubeResponse:
        part: PlaylistItemPart = PlaylistItemPart()
        optional_params: PlaylistItemOptionalParameters = (
            PlaylistItemOptionalParameters(maxResults=max_results)
        )
        request_filter: PlaylistItemFilter = PlaylistItemFilter(playlistId=playlist_id)
        request_schema: YouTubeRequest = YouTubeRequest(
            part=part, filter=request_filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(request_schema)
        find_items_request: dict = self.youtube_client.playlistItems().list(
            **request_dict
        )
        find_items_result: dict = find_items_request.execute()
        return self.parse_youtube_response(find_items_result)

    def find_playlist_items_by_ids(
        self, playlist_item_ids: list[str]
    ) -> YouTubeListResponse:
        request_filter: PlaylistItemFilter = PlaylistItemFilter(id=playlist_item_ids)
        part: PlaylistItemPart = PlaylistItemPart()
        optional_params: PlaylistItemOptionalParameters = (
            PlaylistItemOptionalParameters()
        )
        request_schema: YouTubeRequest = YouTubeRequest(
            part=part, filter=request_filter, optional_parameters=optional_params
        )
        request_dict = self.create_request_dict(request_schema)
        find_video_request: dict = self.youtube_client.playlistItems().list(
            **request_dict
        )
        find_video_result: dict[str, int | str] = find_video_request.execute()
        return self.parse_youtube_list_response(find_video_result)

    def insert_playlist_item(self, create_item: CreatePlaylistItem) -> PlaylistItem:
        insert_playlist_item_request: dict = self.youtube_client.playlistItems().insert(
            part='snippet', body=create_item.model_dump()
        )
        insert_playlist_item_response = insert_playlist_item_request.execute()
        return self.parse_item(insert_playlist_item_response)

    def update_playlist_item(
        self, playlist_id: str, playlist_item_id: str, video_id: str, position: int = 1
    ) -> PlaylistItem:
        update_playlist_item_request: dict = self.youtube_client.playlistItems().update(
            part='snippet',
            body={
                'id': playlist_item_id,
                'snippet': {
                    'playlistId': playlist_id,
                    'position': position,
                    'resourceId': {'kind': 'youtube#video', 'videoId': video_id},
                },
            },
        )
        update_playlist_item_response = update_playlist_item_request.execute()
        return self.parse_item(update_playlist_item_response)

    def delete_playlist_item(self, playlist_item_id: str) -> None:
        delete_playlist_item_request: dict = self.youtube_client.playlistItems().delete(
            id=playlist_item_id,
        )
        delete_playlist_item_request.execute()



from collections.abc import Iterator
from typing import Any, Optional

from oryks_google_oauth import GoogleOAuth, YouTubeScopes
from pydantic import BaseModel

from .models import (
    Caption,
    ChannelSection,
    Comment,
    CommentThread,
    Playlist,
    PlaylistItem,
    Subscription,
    Video,
)
from .resources.channel import YouTubeChannel
from .resources.comment_thread import YouTubeCommentThread
from .resources.playlist import YouTubePlaylist
from .resources.playlist_item import YouTubePlaylistItem
from .resources.search import YouTubeSearch
from .resources.subscription import YouTubeSubscription
from .resources.video import YouTubeVideo
from .resources.activity import YouTubeActivity
from .schemas import (
    CreatePlaylist,
    CreatePlaylistItem,
    InsertChannelSection,
    ThumbnailSetResponse,
    UploadVideo,
    VideoReportReasonSchema,
    YouTubeListResponse,
    YouTubeRequest,
    YouTubeResponse,
)


class YouTube(BaseModel):
    """Provides methods for interacting with the YouTube API.

    This class acts as an interface to the YouTube API, providing methods for interacting with
    the YouTube V3 API.

    Attributes
    ----------
    client_secret_file: str
        The path to the json file containing your authentication information.
    """

    client_secret_file: Optional[str] = None
    authenticated: Optional[bool] = False
    youtube_client: Optional[Any] = None

    def authenticate(self, client_secret_file: Optional[str] = None) -> None:
        """Authenticate the requests made to youtube.

        Used to generate the credentials that are used when authenticating requests to youtube.

        Parameters
        ----------
        client_secret_file: str
            The path to clients secret json file from Google

        Raises
        ------
        ValueError:
            When the client secrets file is not provided
        FileNotFoundError:
            When the secrets file path is not found
        """
        if client_secret_file:
            self.client_secret_file = client_secret_file
        if not self.client_secret_file:
            raise ValueError('The client secret file must be provided.')
        api_service_name: str = 'youtube'
        api_version: str = 'v3'
        credentials_dir: str = '.youtube'
        scopes: list[str] = [
            YouTubeScopes.youtube.value,
            YouTubeScopes.youtube_force_ssl.value,
            YouTubeScopes.youtube_upload.value,
        ]
        oauth: GoogleOAuth = GoogleOAuth(
            secrets_file=self.client_secret_file,
            scopes=scopes,
            api_service_name=api_service_name,
            api_version=api_version,
            credentials_dir=credentials_dir,
        )
        youtube_client = oauth.authenticate_google_server()
        return youtube_client

    def search(self, search_schema: YouTubeRequest) -> YouTubeResponse:
        """Used to search through youtube for videos, playlists and channels.

        Parameters
        ----------
        search_schema: YouTubeRequest
            An instance of YouTubeRequest that contains all the details needed to search.
        Returns
        -------
        YouTubeResponse:
            An instance of YouTubeResponse that has al the search results.
        """
        search: YouTubeSearch = YouTubeSearch(youtube_client=self.youtube_client)
        search_results: YouTubeResponse = search.search(search_schema)
        return search_results

    def get_search_iterator(self, search_schema: YouTubeRequest) -> Iterator:
        """Return an iterator that can be used to iterate over search results.

        Parameters
        ----------
        search_schema: YouTubeRequest
            An instance of YouTubeRequest that contains all the details needed to search.
        Returns
        -------
        Iterator:
            An iterator that can be used to iterate over the search results.
        """
        search: YouTubeSearch = YouTubeSearch(youtube_client=self.youtube_client)
        return search.get_search_iterator(search_schema)

    def find_channel_by_name(self, display_name: str) -> YouTubeResponse:
        """Find a channel's details when given the channel title.

        Parameters
        ----------
        display_name: str
            The name of the youtube channel.
        Returns
        -------
        YouTubeResponse:
            A youtube response consisting of channels whose names match the provided names
        """
        search: YouTubeSearch = YouTubeSearch(youtube_client=self.youtube_client)
        return search.find_channel_by_name(display_name)

    def get_video_ratings(self, video_ids: list[str]) -> YouTubeListResponse:
        """Find out whether or not you liked the videos whose ids are provided.

        You provide it with a list of video ids and for each video it will tell you whether
        or not you have liked that video.

        Parameters
        ----------
        video_ids: list[str]:
            A list of strings of video ids
        Returns
        -------
        list[str]:
            A list of strings showing whether or not you liked a given video.
        """
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.get_video_ratings(video_ids)

    def find_video_by_id(self, video_id: str) -> YouTubeListResponse:
        """Find a single video by providing the video's id.

        Parameters
        ----------
        video_id: str
            The video's id
        Returns
        -------
        Video:
            A Video instance
        """
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.find_video_by_id(video_id)

    def find_videos_by_ids(self, video_ids: list[str]) -> YouTubeListResponse:
        """Find a many videos by providing a list of video ids.

        Parameters
        ----------
        video_ids: list[str]
            A list of video ids
        Returns
        -------
        list[Video]:
            A list of Video instances
        """
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.find_videos_by_ids(video_ids)

    def find_most_popular_video_by_region(
        self, region_code: str = 'US', category_id: str = ''
    ) -> list[Video]:
        """Get the most popular videos in a given region and category.

        Parameters
        ----------
        region_code: str
            A string representing the region code.
        category_id: str
            The category to find. This is a string number i.e '22'
        Returns
        -------
        list[Video]:
            A list of the most popular videos
        """
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.find_most_popular_video_by_region(region_code, category_id)

    def report_video_abuse(self, abuse_report: VideoReportReasonSchema) -> None:
        """Report a video that you deem as harmful or infringing on copyrights.

        Parameters
        ----------
        abuse_report: VideoReportAbuse
            An instance of VideoReportAbuse with all the details needed to report.
        """
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.report_video_abuse(abuse_report)

    def update_video(self, update_req: UploadVideo) -> Video:
        """Updates a video's metadata."""
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.update_video(update_req)

    def delete_video(self, video_id: str) -> None:
        """Deletes a YouTube video."""
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.delete_video(video_id)

    def upload_video(self, upload_req: UploadVideo) -> Video:
        """Upload a YouTube video."""
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.upload_video(upload_req)

    def rate_video(self, video_id: str, rating: str) -> None:
        """Add a like or dislike rating to a video or remove a rating from a video."""
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        video.rate_video(video_id, rating)

    def find_channel_by_id(self, channel_id: str) -> YouTubeListResponse:
        """Find a youtube channel given its id."""
        channel: YouTubeChannel = YouTubeChannel(youtube_client=self.youtube_client)
        return channel.find_channel_by_id(channel_id)

    def find_channels_by_ids(self, channel_ids: list[str]) -> YouTubeListResponse:
        """Find many channels given their ids."""
        channel: YouTubeChannel = YouTubeChannel(youtube_client=self.youtube_client)
        return channel.find_channels_by_ids(channel_ids)

    def find_my_channel(self) -> YouTubeListResponse:
        """Find the details fo your youtube channel."""
        channel: YouTubeChannel = YouTubeChannel(youtube_client=self.youtube_client)
        return channel.find_my_channel()

    def find_channel_playlists(
        self, channel_id: str, max_results: int = 25
    ) -> YouTubeListResponse:
        """Find the playlists for a given channel."""
        playlist: YouTubePlaylist = YouTubePlaylist(youtube_client=self.youtube_client)
        return playlist.find_channel_playlists(channel_id, max_results)

    def find_my_playlists(self) -> YouTubeListResponse:
        """Find the playlists in your channel."""
        playlist: YouTubePlaylist = YouTubePlaylist(youtube_client=self.youtube_client)
        return playlist.find_my_playlists()

    def get_my_playlists_iterator(self, max_results: int = 10) -> Iterator:
        """Get an iterator for iterating through playlists in your channel."""
        playlist: YouTubePlaylist = YouTubePlaylist(youtube_client=self.youtube_client)
        return playlist.get_my_playlists_iterator(max_results)

    def insert_playlist(self, playlist_schema: CreatePlaylist) -> Playlist:
        """Create a new playlist in your channel."""
        playlist: YouTubePlaylist = YouTubePlaylist(youtube_client=self.youtube_client)
        return playlist.insert_playlist(playlist_schema)

    def update_playlist(
        self, playlist_id: str, playlist_schema: CreatePlaylist
    ) -> Playlist:
        """Update a playlist in your channel."""
        playlist: YouTubePlaylist = YouTubePlaylist(youtube_client=self.youtube_client)
        return playlist.update_playlist(playlist_id, playlist_schema)

    def delete_playlist(self, playlist_id: str) -> None:
        """Delete a playlist in your channel."""
        playlist: YouTubePlaylist = YouTubePlaylist(youtube_client=self.youtube_client)
        return playlist.delete_playlist(playlist_id)

    def find_playlist_items(
        self, playlist_id: str, max_results: Optional[int] = 25
    ) -> YouTubeResponse:
        """Find a particular video in your playlist."""
        playlist_item: YouTubePlaylistItem = YouTubePlaylistItem(
            youtube_client=self.youtube_client
        )
        return playlist_item.find_playlist_items(playlist_id, max_results)

    def find_playlist_items_by_ids(
        self, playlist_item_ids: list[str]
    ) -> YouTubeListResponse:
        """Find various playlist videos given their ids."""
        playlist_item: YouTubePlaylistItem = YouTubePlaylistItem(
            youtube_client=self.youtube_client
        )
        return playlist_item.find_playlist_items_by_ids(playlist_item_ids)

    def insert_playlist_item(self, create_item: CreatePlaylistItem) -> PlaylistItem:
        """Add a video to a playlist."""
        playlist_item: YouTubePlaylistItem = YouTubePlaylistItem(
            youtube_client=self.youtube_client
        )
        return playlist_item.insert_playlist_item(create_item)

    def update_playlist_item(
        self, playlist_id: str, playlist_item_id: str, video_id: str, position: int = 1
    ) -> PlaylistItem:
        """Update a given video details in a playlist."""
        playlist_item: YouTubePlaylistItem = YouTubePlaylistItem(
            youtube_client=self.youtube_client
        )
        return playlist_item.update_playlist_item(
            playlist_id, playlist_item_id, video_id, position
        )

    def delete_playlist_item(self, playlist_item_id: str) -> None:
        """Delete a video from a playlist."""
        playlist_item: YouTubePlaylistItem = YouTubePlaylistItem(
            youtube_client=self.youtube_client
        )
        return playlist_item.delete_playlist_item(playlist_item_id)

    def get_video_categories(
        self, region_code: Optional[str] = ''
    ) -> YouTubeListResponse:
        """List all the video categories on youtube."""
        youtube_video = YouTubeVideo(youtube_client=self.youtube_client)
        return youtube_video.get_video_categories(region_code=region_code)

    def find_video_comments(self, request: YouTubeRequest) -> YouTubeResponse:
        """Get a particular video's comments."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.find_video_comments(request)

    def get_comments_iterator(self, request_schema: YouTubeResponse) -> Iterator:
        """Get an iterator for going through a videos comments."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.get_comments_iterator(request_schema)

    def find_all_channel_comments(self, request: YouTubeRequest) -> YouTubeListResponse:
        """Get a particular channels's comments."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.find_all_channel_comments(request)

    def insert_comment(self, video_id: str, comment: str) -> CommentThread:
        """Comment on a given video."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.insert_comment(video_id, comment)

    def get_comment_replies(self, comment_id: str) -> list[Comment]:
        """Get the replies for a given comment."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.get_comment_replies(comment_id)

    def get_comment(self, comment_id: str) -> YouTubeListResponse:
        """Get a comments, given its id."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.get_comment(comment_id)

    def get_comments(self, comment_ids: list[str]) -> YouTubeListResponse:
        """Get various comments given their ids."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.get_comments(comment_ids)

    def reply_to_comment(self, comment_id: str, comment: str) -> Comment:
        """Reply to the given comment."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.reply_to_comment(comment_id, comment)

    def update_comment(self, comment_id: str, comment: str) -> Comment:
        """Update a comments."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.update_comment(comment_id, comment)

    def delete_comment(self, comment_id: str) -> None:
        """Delete a comment."""
        comment_thread: YouTubeCommentThread = YouTubeCommentThread(
            youtube_client=self.youtube_client
        )
        return comment_thread.delete_comment(comment_id)

    def list_channel_activities(self, request: YouTubeRequest) -> YouTubeResponse:
        """List all the activities of the given channel, such as uploads."""
        activity: YouTubeActivity = YouTubeActivity(youtube_client=self.youtube_client)
        return activity.list_channel_activity(request)

    def list_my_activities(self) -> YouTubeListResponse:
        """List all the activities for your channel, such as creating a playlist."""
        activity: YouTubeActivity = YouTubeActivity(youtube_client=self.youtube_client)
        return activity.list_my_activities()

    def list_video_captions(self, video_id: str) -> Caption:
        raise NotImplementedError()

    def insert_video_caption(self, video_id: str) -> None:
        raise NotImplementedError()

    def update_video_caption(self, caption_id: str) -> None:
        raise NotImplementedError()

    def download_video_caption(self, caption_id: str) -> None:
        raise NotImplementedError()

    def delete_video_caption(self, caption_id: str) -> None:
        raise NotImplementedError()

    def upload_channel_banner(self, media_file: str, channel_id: str) -> None:
        raise NotImplementedError()

    def list_channel_sections(self, channel_id: str) -> YouTubeListResponse:
        """List a channel's sections."""
        raise NotImplementedError()

    def list_my_channel_sections(self) -> YouTubeListResponse:
        """List your channel sections."""
        raise NotImplementedError()

    def insert_channel_section(self, request: InsertChannelSection) -> ChannelSection:
        """Insert a new channle section."""
        raise NotImplementedError()

    def update_channel_section(self, channel_section_id: str) -> None:
        """Update a channel section."""
        raise NotImplementedError()

    def delete_channel_section(self, channel_section_id: str) -> None:
        """Delete a channel section."""
        raise NotImplementedError()

    def list_languages(self, language: str = 'en_US') -> YouTubeListResponse:
        """List all the languages youtube supports."""
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.list_languages(language)

    def list_regions(self, language: str = 'en_US') -> YouTubeListResponse:
        """List all the regions youtube supports."""
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.list_regions(language)

    def set_watermark(self, channel_id: str) -> None:
        """Set a video's watermarks."""
        raise NotImplementedError()

    def unset_watermark(self, channel_id: str) -> None:
        """Remove a video's watermark."""
        raise NotImplementedError()

    def list_video_abuse_report_reasons(self) -> YouTubeListResponse:
        """List reasons that can be used to report a video."""
        video: YouTubeVideo = YouTubeVideo(youtube_client=self.youtube_client)
        return video.list_video_abuse_report_reasons()

    def set_video_thumbnail(
        self, video_id: str, thumbnail: str
    ) -> ThumbnailSetResponse:
        """Set a video's thumbnail."""
        raise NotImplementedError()

    def list_channel_subscriptions(self, request: YouTubeRequest) -> YouTubeResponse:
        """List the given channel's subscriptions."""
        subscription: YouTubeSubscription = YouTubeSubscription(
            youtube_client=self.youtube_client
        )
        return subscription.list_channel_subscriptions(request)

    def list_my_subscriptions(self) -> YouTubeResponse:
        """List the channels I subscribe to."""
        subscription: YouTubeSubscription = YouTubeSubscription(
            youtube_client=self.youtube_client
        )
        return subscription.list_my_subscriptions()

    def do_i_subscribe(self, channel_id: str) -> bool:
        """Check if I subscribe to the given channel."""
        subscription: YouTubeSubscription = YouTubeSubscription(
            youtube_client=self.youtube_client
        )
        return subscription.do_i_subscribe(channel_id)

    def is_subscribed(self, channel_id: str, user_channel: str) -> bool:
        """Check if one channel subscribes to another channel.

        Parameters
        ----------
        channel_id: str
            The channel id of the channel to which subscription is being confirmed.
        user_channel: str
            The channel id of the channel to check if its subscribed.
        Returns
        -------
        bool:
            Whether user_channel subscribes to channel_id
        """
        subscription: YouTubeSubscription = YouTubeSubscription(
            youtube_client=self.youtube_client
        )
        return subscription.is_subscribed(
            channel_id=channel_id, user_channel=user_channel
        )

    def subscribe_to_channel(self, channel_id: str) -> Subscription:
        """Subscribe to a youtube channel.

        Parameters
        ----------
        channel_id: str
            The id of the channel you want to subscribe to.
        Returns
        -------
        Subscription:
            An instance of subscription with all the details on the channel subscription.
        """
        subscription: YouTubeSubscription = YouTubeSubscription(
            youtube_client=self.youtube_client
        )
        return subscription.subscribe_to_channel(channel_id=channel_id)

    def unsubscribe_from_channel(self, subscription_id: str) -> None:
        """Unsubscribe from a youtube channel.

        Parameters
        ----------
        subscription_id: str
            The subscription id. You can get this by listing your subscriptions.
        """
        subscription: YouTubeSubscription = YouTubeSubscription(
            youtube_client=self.youtube_client
        )
        return subscription.unsubscribe_from_channel(subscription_id)

    def list_activities(self) -> dict:
        """List your activities on youtube including subscriptions, likes."""
        raise NotImplementedError()
